# coding: utf-8
"""
PYUNO_LOGLEVEL     Valid values are

    * NONE - nothing is logged
    * CALL - the method name of every call is logged
    * ARGS - additionally, the arguments of every call are logged 

NONE is default

PYUNO_LOGTARGET 

    * stdout - logs to stdout (doesnt work on windows within OpenOffifce.org)
    * stderr - logs to stderr (doesnt work on windows within OpenOffifce.org)
    * file-url-prefix(relative urls allowed) - logs to files, which start with
      this string. The pid of the process is appended to the string
      (e.g. file:///c:/temp/bla will write to c:\temp\bla.235 if 235 is the
      pid of the current process)
"""

import uno
import unohelper

from com.sun.star.task import XJob
from com.sun.star.document import DocumentEvent
from com.sun.star.lang import XServiceInfo

class JobExecutor(XJob, XServiceInfo, unohelper.Base):
    IMPLE_NAME = "hellug.sgeorgaras.ancientgreek.autoload"
    DOCUMENT_EVENTS = ("OnNew", "OnLoad") 
    #DOCUMENT_EVENTS = ("OnCreate", "OnNew", "OnLoad") 
    """ DOCUMENT_EVENTS = ("OnCreate", 
        "OnLoadFinished", "OnNew", "OnLoad", 
        "OnSave", "OnSaveAs", "OnSaveAsDone", 
        "OnPrepareunload", "OnFocus", "OnUnfocus", 
        "OnPrint", "OnModifyChanged", 
        "OnCopyTo", "OnCopyToDone", 
        "OnViewCreated", "OnPrepareViewClosing", 
        "OnVisAreaChanged", "OnCreate", "OnLoadFinished", 
        "OnSaveAsFailed", "OnSaveFailed", "OnCopyToFailed", 
        "TitleChanged", "OnMailMerge", "OnPageCountChange") """
    APPLICATION_EVENTS = ("OnStartApp", "OnCloseApp", 
        "OnUnload", "OnViewClosed")
    
    def __init__(self, ctx, *args):
        self.ctx = ctx
    # XServiceInfo
    def getImplementationName(self):
        return IMPLE_NAME
    def supportsService(self, name):
        return name == IMPLE_NAME
    def getSupportedServiceNames(self):
        return IMPLE_NAME, 
    def check_context(self, doc, context):
        if len(context) == 0:
            return True
        
        mod_manager = self.ctx.getServiceManager().createInstanceWithContext(
            "com.sun.star.frame.ModuleManager", self.ctx)
        identifier = mod_manager.identify(doc)
        return identifier in context
   
    def createUnoService(self,serviceName):
        """ gets a service from Uno """
        sm = uno.getComponentContext().ServiceManager
        result = sm.createInstance(serviceName)
        #if not result:
        #    raise RuntimeException("\nService not available :\n" + serviceName, uno.getComponentContext())
        return result

    def execute(self, args):
        """ Event executed. """
        event_name = ""
        doc = None
        url = ""
        context = []
        
        for arg in args:
            name = arg.Name
            #print(name)
            if name == "JobConfig": # Arguments
                options = arg.Value
                for option in options:
                    name = option.Name
                    if name == "URL":
                        url = option.Value
                        url.replace("&amp;","&")
                    elif name == "Context":
                        context = option.Value.split(",")
            elif name == "Environment":
                envs = arg.Value
                #print(envs)
                for env in envs:
                    name = env.Name
                    if name == "EnvType" and env.Value == "DOCUMENTEVENT":
                        pass#print(env.Value)
                    elif name == "Model":
                        doc = env.Value
                    elif name == "EventName":
                        event_name = env.Value
        #
        valid = True
        if event_name in self.DOCUMENT_EVENTS:
            if not self.check_context(doc, context):
                valid = False
        else:
            # elif event_name in self.APPLICATION_EVENTS:
            doc = None
        
        if event_name == "OnStartApp":
            print('==== AncientGreek (autoload.py) : Executing AncientGreek.modAccelerator.InitProgram')
            url = "vnd.sun.star.script:AncientGreek.modAccelerator.InitProgram?language=Basic&location=application"
            _mspf = self.createUnoService("com.sun.star.script.provider.MasterScriptProviderFactory")
            sp = _mspf.createScriptProvider("") 
            script = sp.getScript(url)
            """ ev = DocumentEvent(doc, event_name, None, None) """
            ev=None
            a=None
            b=None
            script.invoke((ev,), a, b)
            return

        if valid:
            print('==== AncientGreek (autoload.py) : Run document Auto function?')
            # execute macro
            if not (doc is None):
                if not (doc.supportsService('com.sun.star.script.BasicIDE')):
                    loc=doc.getLocation()
                    if not (loc.startswith("vnd.sun.star.help")):
                        url = "vnd.sun.star.script:AncientGreek.IbycusIcon.auto?language=Basic&location=application"
                        sp = doc.getScriptProvider()
                        script = sp.getScript(url)
                        """ ev = DocumentEvent(doc, event_name, None, None) """
                        ev=None
                        a=None
                        b=None
                        print('==== AncientGreek (autoload.py) : Executing AncientGreek.IbycusIcon.auto')
                        script.invoke((ev,), a, b)
                    else:
                        print('==== AncientGreek (autoload.py) : Document Location = vnd.sun.star.help - Exiting!!!')
                else:
                    print('==== AncientGreek (autoload.py) : Supports Service = com.sun.star.script.BasicIDE - Exiting!!!')
            else:
                print('==== AncientGreek (autoload.py) : DOC is None!!! - Exiting!!!')


g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(
    JobExecutor,
    "hellug.sgeorgaras.ancientgreek.autoload", ("hellug.sgeorgaras.ancientgreek.autoload",),)

